package handlers

import (
	"context"
	"errors"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/go-orusapi/database"

	"orus.io/orus-io/rednerd/auth"
	"orus.io/orus-io/rednerd/models"
	op_auth "orus.io/orus-io/rednerd/restapi/operations/auth"
)

// NewAuthRefreshTokenHandler ...
func NewAuthRefreshTokenHandler(db *sqlx.DB, auth *auth.Auth) *AuthRefreshToken {
	return &AuthRefreshToken{db, auth}
}

// AuthRefreshToken handles /auth/renew_token.
type AuthRefreshToken struct {
	db   *sqlx.DB
	auth *auth.Auth
}

func (h *AuthRefreshToken) handle(
	ctx context.Context,
	log zerolog.Logger,
	principal *models.Principal,
) (string, error) {
	db := database.NewSQLHelper(ctx, h.db, log)
	q := squirrel.Select(models.UserDBColumns...).
		From(models.UserDBTable).
		Where(squirrel.Eq{models.UserDBUsernameColumn: principal.Subject})
	var users []models.UserDB
	if err := db.Select(&users, q); err != nil {
		return "", err
	}
	if len(users) == 0 {
		return "", errors.New("user does not exist in the database")
	}
	q = squirrel.Select(models.AccountDBColumns...).
		From(models.AccountDBTable).
		Where(squirrel.Eq{models.AccountDBNameColumn: principal.Subject})
	var accounts []models.AccountDB
	if err := db.Select(&accounts, q); err != nil {
		return "", err
	}
	if len(users) == 0 {
		return "", errors.New("account does not exist in the database")
	}
	account := accounts[0]
	if !account.IsUser {
		return "", errors.New("the account but be a user")
	}

	token, _, err := h.auth.MakeToken(*principal, false)

	return token, err
}

// Handle handles the request.
func (h *AuthRefreshToken) Handle(
	params op_auth.RefreshTokenParams, principal *models.Principal,
) middleware.Responder {
	ctx := params.HTTPRequest.Context()
	log := zerolog.Ctx(ctx)
	token, err := h.handle(ctx, *log, principal)
	if err != nil {
		return op_auth.NewRefreshTokenDefault(500).WithPayload(models.FromError(err))
	}

	return op_auth.NewRefreshTokenOK().WithPayload(token)
}
